// ignore_for_file: public_member_api_docs, sort_constructors_first
// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:io';

import 'package:wyatt_cli_toolbox/src/spinners/spinners.dart';
import 'package:wyatt_cli_toolbox/src/stylish/stylish.dart';

typedef StyleFunction = String Function(String);

class Theme {
  final String inputPrefix;
  final String inputSuffix;
  final String successPrefix;
  final String successSuffix;
  final String errorPrefix;
  final String hiddenPrefix;
  final StyleFunction messageStyle;
  final StyleFunction errorStyle;
  final StyleFunction hintStyle;
  final StyleFunction valueStyle;
  final StyleFunction defaultStyle;

  final String activeItemPrefix;
  final String inactiveItemPrefix;
  final StyleFunction activeItemStyle;
  final StyleFunction inactiveItemStyle;

  final String checkedItemPrefix;
  final String uncheckedItemPrefix;

  final String pickedItemPrefix;
  final String unpickedItemPrefix;

  final bool showActiveCursor;

  final SpinnerType spinner;

  const Theme({
    required this.inputPrefix,
    required this.inputSuffix,
    required this.successPrefix,
    required this.successSuffix,
    required this.errorPrefix,
    required this.hiddenPrefix,
    required this.messageStyle,
    required this.errorStyle,
    required this.hintStyle,
    required this.valueStyle,
    required this.defaultStyle,
    required this.activeItemPrefix,
    required this.inactiveItemPrefix,
    required this.activeItemStyle,
    required this.inactiveItemStyle,
    required this.checkedItemPrefix,
    required this.uncheckedItemPrefix,
    required this.pickedItemPrefix,
    required this.unpickedItemPrefix,
    required this.showActiveCursor,
    required this.spinner,
  });

  Theme copyWith({
    String? inputPrefix,
    String? inputSuffix,
    String? successPrefix,
    String? successSuffix,
    String? errorPrefix,
    String? hiddenPrefix,
    StyleFunction? messageStyle,
    StyleFunction? errorStyle,
    StyleFunction? hintStyle,
    StyleFunction? valueStyle,
    StyleFunction? defaultStyle,
    String? activeItemPrefix,
    String? inactiveItemPrefix,
    StyleFunction? activeItemStyle,
    StyleFunction? inactiveItemStyle,
    String? checkedItemPrefix,
    String? uncheckedItemPrefix,
    String? pickedItemPrefix,
    String? unpickedItemPrefix,
    bool? showActiveCursor,
    SpinnerType? spinner,
  }) {
    return Theme(
      inputPrefix: inputPrefix ?? this.inputPrefix,
      inputSuffix: inputSuffix ?? this.inputSuffix,
      successPrefix: successPrefix ?? this.successPrefix,
      successSuffix: successSuffix ?? this.successSuffix,
      errorPrefix: errorPrefix ?? this.errorPrefix,
      hiddenPrefix: hiddenPrefix ?? this.hiddenPrefix,
      messageStyle: messageStyle ?? this.messageStyle,
      errorStyle: errorStyle ?? this.errorStyle,
      hintStyle: hintStyle ?? this.hintStyle,
      valueStyle: valueStyle ?? this.valueStyle,
      defaultStyle: defaultStyle ?? this.defaultStyle,
      activeItemPrefix: activeItemPrefix ?? this.activeItemPrefix,
      inactiveItemPrefix: inactiveItemPrefix ?? this.inactiveItemPrefix,
      activeItemStyle: activeItemStyle ?? this.activeItemStyle,
      inactiveItemStyle: inactiveItemStyle ?? this.inactiveItemStyle,
      checkedItemPrefix: checkedItemPrefix ?? this.checkedItemPrefix,
      uncheckedItemPrefix: uncheckedItemPrefix ?? this.uncheckedItemPrefix,
      pickedItemPrefix: pickedItemPrefix ?? this.pickedItemPrefix,
      unpickedItemPrefix: unpickedItemPrefix ?? this.unpickedItemPrefix,
      showActiveCursor: showActiveCursor ?? this.showActiveCursor,
      spinner: spinner ?? this.spinner,
    );
  }

  static Theme defaultTheme = colorfulTheme;

  static final colorfulTheme = Theme(
    inputPrefix: '?'.padRight(2).green(),
    inputSuffix: (Platform.isWindows ? '>' : '\u276F').padLeft(2).grey(),
    successPrefix: (Platform.isWindows ? 'v' : '\u2714').padRight(2).green(),
    successSuffix: '•'.padLeft(2).grey(),
    errorPrefix: (Platform.isWindows ? 'x' : '\u2718').padRight(2).red(),
    hiddenPrefix: '*****',
    messageStyle: (String message) => message.bold(),
    errorStyle: (String error) => error.red(),
    hintStyle: (String hint) => '($hint)'.grey(),
    valueStyle: (String value) => value.green(),
    defaultStyle: (String value) => value.cyan(),
    activeItemPrefix: (Platform.isWindows ? '>' : '\u276F').green(),
    inactiveItemPrefix: ' ',
    activeItemStyle: (String item) => item.cyan(),
    inactiveItemStyle: (String item) => item,
    checkedItemPrefix: (Platform.isWindows ? 'v' : '\u2714').green(),
    uncheckedItemPrefix: ' ',
    pickedItemPrefix: (Platform.isWindows ? '>' : '\u276F').green(),
    unpickedItemPrefix: ' ',
    showActiveCursor: false,
    spinner: SpinnerType.arrow,
  );
}
